#!/usr/bin/perl
#    Diffpicture v1.0 - Calculates the difference between two images
#    Copyright (c) 2002 Arne "Heinz99" Bochem <webmaster@heinz99.de>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# TODO
# - Move images in x/y dir to check for moving cam. o_O
# - Do something against more/less light in the diffs.

use strict;
use GD;

# No check for image dimensions. Users don't make errors, do they?
# Ok, you are right, we'll do the check.
my $img1 = new GD::Image->newFromPng($ARGV[0]);
my $img2 = new GD::Image->newFromPng($ARGV[1]);
#my $img1 = new GD::Image->newFromJpeg($ARGV[0]);
#my $img2 = new GD::Image->newFromJpeg($ARGV[1]);
my ($height1, $width1) = $img1->getBounds();
my ($height2, $width2) = $img2->getBounds();
(($height1 == $height2) && ($width1 == $width2)) or die('Image size mismatch.');

# Make two pixel arrays.
my @pixels1 = getImagedata($img1, $height1, $width1);
my @pixels2 = getImagedata($img2, $height2, $width2);
my $pxnum = $height1 * $width1;
my $diff;

# Compare and add diffs.
for (my $count = 0; $count < $pxnum; $count++)
{
	my $p1 = pop(@pixels1);
	my $p2 = pop(@pixels2);
	($p1 && $p2) or next;
	$diff += pxDiff(@{$p1}, @{$p2});
}

# Ok, we are almost done. Divide the difference by the number of pixels.
$diff /= $pxnum;

# Done.
print($diff."%\n");

sub getImagedata ($$$) # Just getPixel with a for().
{
	my ($img, $x, $y) = @_;
	my @imagedata;
	for (my $countx = 0; $countx < $x; $countx++)
	{
		for (my $county = 0; $county < $y; $county++)
		{
			my @col = getPixel($img, $countx, $county);
			push(@imagedata, \@col);
		}
	}
	return(@imagedata);
}

sub getPixel ($$$) # Make this code nicer.
{
	my ($img, $x, $y) = @_;
	return($img->rgb($img->getPixel($x, $y)));
}

sub pxDiff ($$$$$$) # MAKE MONEY FAST $$$$$$
{
	my ($r1, $g1, $b1, $r2, $g2, $b2) = @_;
	return(((abs($r1 - $r2) + abs($g1 - $g2) + abs($b1 - $b2))/(3*255))*100);
}
